
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#ifndef __IDO_ENUM_TYPES_H__
#define __IDO_ENUM_TYPES_H__

#include <glib-object.h>

G_BEGIN_DECLS
/* enumerations from "idorange.h" */
GType ido_range_style_get_type (void) G_GNUC_CONST;
#define IDO_TYPE_RANGE_STYLE (ido_range_style_get_type())
/* enumerations from "idoscalemenuitem.h" */
GType ido_scale_menu_item_style_get_type (void) G_GNUC_CONST;
#define IDO_TYPE_SCALE_MENU_ITEM_STYLE (ido_scale_menu_item_style_get_type())
/* enumerations from "idotimeline.h" */
GType ido_timeline_direction_get_type (void) G_GNUC_CONST;
#define IDO_TYPE_TIMELINE_DIRECTION (ido_timeline_direction_get_type())
GType ido_timeline_progress_type_get_type (void) G_GNUC_CONST;
#define IDO_TYPE_TIMELINE_PROGRESS_TYPE (ido_timeline_progress_type_get_type())
G_END_DECLS

#endif /* !__IDO_ENUM_TYPES_H__ */

/* Generated data ends here */

